@@ -17,22 +17,22 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "kreversiview.h"
+#include <kreversiview.h>
 
-KReversiView::KReversiView(KReversiGame* game, QWidget *parent) :
+#include <KLocalizedString>
+#include <KStandardDirs>
+
+#include <colorscheme.h>
+
+KReversiView::KReversiView(KReversiGame* game, QWidget *parent, KgThemeProvider *provider) :
     KgDeclarativeView(parent), m_delay(ANIMATION_SPEED_NORMAL), m_game(0),
-    m_demoMode(false), m_showLastMove(false), m_showLegalMoves(false),
-    m_showLabels(false), m_maxDelay(0)
+    m_showLastMove(false), m_showLegalMoves(false),
+    m_showLabels(false), m_provider(provider)
 {
-    m_delayTimer.setSingleShot(true);
-    connect(&m_delayTimer, SIGNAL(timeout()), this, SLOT(slotOnDelay()));
-    
-    qmlRegisterType<ColorScheme>("ColorScheme", 1, 0, "ColorScheme");
-
-    m_provider = new KgThemeProvider();
-    m_provider->discoverThemes("appdata", QLatin1String("pics"));
     m_provider->setDeclarativeEngine("themeProvider", engine());
 
+    qmlRegisterType<ColorScheme>("ColorScheme", 1, 0, "ColorScheme");
+
     QString path =
         KStandardDirs::locate("appdata", QLatin1String("qml/Table.qml"));
     setSource(QUrl::fromLocalFile(path));
@@ -47,14 +47,14 @@ KReversiView::KReversiView(KReversiGame* game, QWidget *parent) :
 
 void KReversiView::setGame(KReversiGame *game)
 {
-    // disconnect signals from previous game if it exists,
+    // disconnect signals from previous game if they exist,
     // we are not interested in them anymore
     if (m_game) {
         disconnect(m_game, SIGNAL(boardChanged()), this, SLOT(updateBoard()));
-        disconnect(m_game, SIGNAL(moveFinished()), this, SLOT(slotGameMoveFinished()));
-        disconnect(m_game, SIGNAL(gameOver()), this, SLOT(slotGameOver()));
-        disconnect(m_game, SIGNAL(computerCantMove()), this, SLOT(slotComputerCantMove()));
-        disconnect(m_game, SIGNAL(playerCantMove()), this, SLOT(slotPlayerCantMove()));
+        disconnect(m_game, SIGNAL(moveFinished()), this, SLOT(gameMoveFinished()));
+        disconnect(m_game, SIGNAL(gameOver()), this, SLOT(gameOver()));
+        disconnect(m_game, SIGNAL(whitePlayerCantMove()), this, SLOT(whitePlayerCantMove()));
+        disconnect(m_game, SIGNAL(blackPlayerCantMove()), this, SLOT(blackPlayerCantMove()));
         delete m_game;
     }
 
@@ -62,22 +62,23 @@ void KReversiView::setGame(KReversiGame *game)
 
     if (m_game) {
         connect(m_game, SIGNAL(boardChanged()), this, SLOT(updateBoard()));
-        connect(m_game, SIGNAL(moveFinished()), this, SLOT(slotGameMoveFinished()));
-        connect(m_game, SIGNAL(gameOver()), this, SLOT(slotGameOver()));
-        connect(m_game, SIGNAL(computerCantMove()), this, SLOT(slotComputerCantMove()));
-        connect(m_game, SIGNAL(playerCantMove()), this, SLOT(slotPlayerCantMove()));
+        connect(m_game, SIGNAL(moveFinished()), this, SLOT(gameMoveFinished()));
+        connect(m_game, SIGNAL(gameOver()), this, SLOT(gameOver()));
+        connect(m_game, SIGNAL(whitePlayerCantMove()), this, SLOT(whitePlayerCantMove()));
+        connect(m_game, SIGNAL(blackPlayerCantMove()), this, SLOT(blackPlayerCantMove()));
+
+        m_game->setDelay(m_delay);
     }
 
-
-    m_hint = KReversiPos();
-    m_demoMode = false;
+    m_hint = KReversiMove();
 
     updateBoard();
 }
 
-void KReversiView::setChipsPrefix(const QString &chipsPrefix)
+void KReversiView::setChipsPrefix(ChipsPrefix chipsPrefix)
 {
-    m_qml_root->setProperty("chipsImagePrefix", chipsPrefix);
+    m_qml_root->setProperty("chipsImagePrefix",
+                            Utils::chipPrefixToString(chipsPrefix));
 }
 
 void KReversiView::setShowBoardLabels(bool show)
@@ -104,6 +105,9 @@ void KReversiView::setAnimationSpeed(int speed)
 
     m_delay = value;
 
+    if (m_game)
+        m_game->setDelay(value);
+
     m_qml_root->setProperty("chipsAnimationTime", value);
 }
 
@@ -114,31 +118,20 @@ KReversiView::~KReversiView()
 
 void KReversiView::updateBoard()
 {
-    for (int i = 0; i < 8; i++)
+    for (int i = 0; i < 8; i++) {
         for (int j = 0; j < 8; j++) {
-            QMetaObject::invokeMethod(m_qml_root, "setPreAnimationTicks",
-                                    Q_ARG(QVariant, i),
-                                    Q_ARG(QVariant, j),
-                                    Q_ARG(QVariant, 0));
+            QMetaObject::invokeMethod(m_qml_root, "setPreAnimationTime",
+                                      Q_ARG(QVariant, i),
+                                      Q_ARG(QVariant, j),
+                                      Q_ARG(QVariant, m_game ? m_game->getPreAnimationDelay(KReversiPos(i, j)) : 0));
         }
-        
-    m_maxDelay = 0;
-    if (m_game) {
-        PosList changed_chips = m_game->changedChips();
-        for (int i = 1; i < changed_chips.size(); i++) { //i == 0 is new chip it don't need animation time
-            QMetaObject::invokeMethod(m_qml_root, "setPreAnimationTicks",
-                                    Q_ARG(QVariant, changed_chips[i].row),
-                                    Q_ARG(QVariant, changed_chips[i].col),
-                                    Q_ARG(QVariant, i - 1));
-        }
-        m_maxDelay = qMax(0, (int)(changed_chips.size()) - 2);
     }
         
     for (int i = 0; i < 8; i++)
         for (int j = 0; j < 8; j++) {
             QString new_state = "";
             if (m_game) // showing empty board if has no game
-                switch (m_game->chipColorAt(i, j)) {
+                switch (m_game->chipColorAt(KReversiMove(NoColor, i, j))) {
                 case Black:
                     new_state = "Black";
                     break;
@@ -171,7 +164,7 @@ void KReversiView::updateBoard()
         }
 
     if (m_game && m_showLegalMoves) {
-        PosList possible_moves = m_game->possibleMoves();
+        MoveList possible_moves = m_game->possibleMoves();
         for (int i = 0; i < possible_moves.size(); i++) {
             QMetaObject::invokeMethod(m_qml_root, "setLegal",
                                       Q_ARG(QVariant, possible_moves.at(i).row),
@@ -194,7 +187,7 @@ void KReversiView::updateBoard()
     }
 
     if (m_game && m_showLastMove) {
-        KReversiPos lastmove = m_game->getLastMove();
+        KReversiMove lastmove = m_game->getLastMove();
         if (lastmove.isValid())
             QMetaObject::invokeMethod(m_qml_root, "setLastMove",
                                       Q_ARG(QVariant, lastmove.row),
@@ -218,81 +211,47 @@ void KReversiView::setShowLegalMoves(bool show)
 void KReversiView::slotHint()
 {
     if (!m_game) {
-        m_hint = KReversiPos();
+        m_hint = KReversiMove();
         return;
     }
 
-    if (m_game->isComputersTurn())
-        return;
-
     m_hint = m_game->getHint();
     updateBoard();
 }
 
-void KReversiView::setDemoMode(bool state)
-{
-    if (!m_game)
-        return;
-
-    if (m_game->isGameOver())
-        return;
-
-    m_demoMode = state;
-
-    if (!m_game->isComputersTurn())
-        m_game->startNextTurn(m_demoMode);
-}
-
 void KReversiView::onPlayerMove(int row, int col)
 {
     if (!m_game)
         return;
 
-    // user moves not allowed in demo mode
-    if (m_demoMode)
-        return;
-    
-    if (m_delayTimer.isActive())
-        return;
-
-    if (m_game->isComputersTurn())
-        return;
-
-    m_game->makePlayerMove(row, col, false);
+    emit userMove(KReversiPos(row, col));
 }
 
-void KReversiView::slotGameMoveFinished()
+
+void KReversiView::gameMoveFinished()
 {
-    m_hint = KReversiPos();
+    m_hint = KReversiMove();
     updateBoard();
-    emit moveFinished();
-    m_delayTimer.setInterval(m_delay * (m_maxDelay + 1));
-    m_delayTimer.start();
 }
 
-void KReversiView::slotGameOver()
+void KReversiView::gameOver()
 {
-    m_demoMode = false;
 }
 
-void KReversiView::slotComputerCantMove()
+void KReversiView::whitePlayerCantMove()
 {
+    // TODO: use Computer, You and Opponent instead in message
     QMetaObject::invokeMethod(m_qml_root, "showPopup",
                               Q_ARG(QVariant,
-                                    i18n("Computer can not move. It is your turn again.")));
+                                    i18n("White can not perform any move. It is black turn again.")));
     updateBoard();
 }
 
-void KReversiView::slotPlayerCantMove()
+void KReversiView::blackPlayerCantMove()
 {
+    // TODO: use Computer, You and Opponent instead in message
     QMetaObject::invokeMethod(m_qml_root, "showPopup",
                               Q_ARG(QVariant,
-                                    i18n("You can not perform any move. Computer takes next turn now.")));
+                                    i18n("Black can not perform any move. It is white turn again.")));
     updateBoard();
 }
-
-void KReversiView::slotOnDelay()
-{
-    if (m_game)
-        m_game->startNextTurn(m_demoMode);
-}
